/// <reference path="JsDiagram-vsdoc.js" />

var Diagram = MindFusion.Diagramming.Diagram;
var Events = MindFusion.Diagramming.Events;
var Behavior = MindFusion.Diagramming.Behavior;
var ContainerNode = MindFusion.Diagramming.ContainerNode;
var ShapeNode = MindFusion.Diagramming.ShapeNode;
var ImageAlign = MindFusion.Drawing.ImageAlign;
var Thickness = MindFusion.Drawing.Thickness;

var Rect = MindFusion.Drawing.Rect;

var TreeMapLayout = MindFusion.Graphs.TreeMapLayout;

var diagramView, diagram, scale;

document.addEventListener("DOMContentLoaded", function ()
{
	// create a DiagramView component that wraps the "diagram" canvas
	diagramView = MindFusion.Diagramming.DiagramView.create(document.getElementById("diagram"));
	diagramView.behavior = Behavior.Modify;
	
	diagram = diagramView.diagram;
	diagram.defaultShape = 'Rectangle';
	diagram.bounds = new Rect(0, 0, 250, 195);
	diagram.backgroundImageUrl = "images\\earth.jpg";
	diagram.backgroundImageAlign = ImageAlign.Tile;

	diagram.addEventListener(Events.controlLoaded, function (sender) { sender.zoomToFit(); });

	scale = ['rgb(3, 148, 71)', 'rgb(2, 147, 116)', 'rgb(45, 143, 172)'];

	createTreeMap();

});

function createTreeMap()
{		
	  diagram.clearAll();

		// pretend we are calling a web service
		var xhttp = new XMLHttpRequest();
		xhttp.onreadystatechange = function ()
		{
			if (this.readyState == 4 && this.status == 200)
			{
				var parser = new DOMParser();
				var xmlDoc = parser.parseFromString(xhttp.responseText, "text/xml");
				var cnt = 0;

				var countries = xmlDoc.getElementsByTagName('country');
				var container = diagram.factory.createContainerNode(0, 0, 10, 10);
				container.text = "Lithium production";
				container.textColor = 'rgb(225, 225, 225)';
				container.captionBackBrush = 'rgb(102, 121, 180)';
				container.captionHeight = 10;
				container.style.fontSize = 6;
				
				for (var i = 0; i < countries.length; i++)
				{
					
					var country = countries[i];
					var shape = diagram.factory.createShapeNode(new Rect(0, 0, 10, 10));
					var amount = parseInt(country.attributes["production"].value);
					
					shape.tag = amount;
					shape.text = country.attributes["name"].value;
					shape.style.fontSize = 5;
					shape.textColor = 'rgb(225, 225, 225)';
					shape.tooltip = shape.text + "<br />Production (tonnes): ~" + amount;
					shape.imageLocation = "images\\" + shape.text + ".png";
					shape.imageAlign = ImageAlign.Center;
					
					shape.imagePadding = new Thickness(0, 0, 0, 20);
					shape.id = cnt++;
					container.add(shape);
					
				}

				assignWeights();
				assignColors();
				arrangeDiagram();
			}
		};
		
		xhttp.open("GET", "Data.xml", true);
		xhttp.send();
}

function assignWeights()
{
	for (var i = 0; i < diagram.nodes.length; i++)
		{
			var node = diagram.nodes[i];
			// Ignore containers and container captions
			if (node instanceof ContainerNode || node.locked)
				continue;

			if (node instanceof ShapeNode)
			{
				var value = node.tag;
				
				if (value == null)
					continue;
				node.weight = value;

			}
		}
}

function assignColors()
{
	var largestProducer = new Map();

	for (var i = 0; i < diagram.nodes.length; i++)
	{
		var node = diagram.nodes[i];
		if (node instanceof ContainerNode)
		{
			var production = 0.0;
		
			for (var j = 0; j < node.children.length; j++)
			{
				var child = node.children[j];
				var value = child.tag;
				if (value == null)
					continue;

				production = Math.max(production, value);
				
			}

			largestProducer.set(node, production);
			
		}
	}

	for (var i = 0; i < diagram.nodes.length; i++)
	{
		var node = diagram.nodes[i];
		// Ignore containers and container captions
		if (node instanceof ContainerNode || node.locked)
			continue;

		if (node instanceof ShapeNode)
		{
			var value = node.tag;
			if (value == null)
				continue;

		
			var total = largestProducer.get(node.container);
			
			node.brush = getScaleColor(value, total);
		
		}
	}
}

function getScaleColor(value, total)
{
	var factor = value / total;
	if (factor < 0)
		factor = 0;
	if (factor > 1)
		factor = 1;

	var step = 1.0 / (scale.length - 1);
	var start = scale[Math.floor(factor / step)];
	var end = scale[Math.min(scale.length - 1, Math.floor(factor / step) + 1)];

	var factorBase = factor - factor % step;

	start = start.substring(4, start.length - 1)
         .replace(/ /g, '')
         .split(',');
	end = end.substring(4, end.length - 1)
         .replace(/ /g, '')
         .split(',');

	return 'rgb(' +
            (+start[0] + Math.floor(((factor - factorBase) * (end[0] - start[0]) / step))).toString() + "," +
            (+start[1] + Math.floor(((factor - factorBase) * (end[1] - start[1]) / step))).toString() + "," +
            (+start[2] + Math.floor(((factor - factorBase) * (end[2] - start[2]) / step))).toString() + ")";
}

function arrangeDiagram()
{
	var layout = new TreeMapLayout();
	layout.padding = 0;
	layout.containerPadding = 1;
	layout.layoutArea = new Rect(0, 0, 350, 200);
	diagram.arrange(layout);
	diagram.resizeToFitItems();

	// Remove the text of smaller nodes
	for (var i = 0; i < diagram.nodes.length; i++)
	{
		var node = diagram.nodes[i];
		if (!node instanceof ShapeNode || node.locked)
			continue;

		if (node.bounds.width < 10 || node.bounds.height < 10)
			node.text = "";
	}

	// Resize caption nodes
	for (var i = 0; i < diagram.nodes.length; i++)
	{
		var node = diagram.nodes[i];
		if (node.locked && node.masterNode)
		{
			var bounds = node.bounds;
			bounds.width = node.masterNode.bounds.width;
			bounds.height = Math.min(30, node.masterNode.bounds.height);
			node.bounds = bounds;
		}
	}
	
	zoom();
}

function zoom()
{
	diagramView.zoomFactor = 100;
	var cb = diagram.getContentBounds();
	var tl = diagramView.docToClient(cb.topLeft());
	var rc = new Rect(tl.x, tl.y, cb.width, cb.height);

	var ctrlWidth = diagram.bounds.width;
	var ctrlHeight = diagram.bounds.height;
	var cntWidth = rc.width;
	var cntHeight = rc.height;

	if (ctrlWidth > 0 && ctrlHeight > 0 && cntWidth > 0 && cntHeight > 0)
	{
		var rx = cntWidth / ctrlWidth;
		var ry = cntHeight / ctrlHeight;
		var ratio = Math.max(rx, ry);

		diagramView.scrollX = rc.x;
		diagramView.scrollY = rc.y;

		var newZoom = 100.0 / ratio;
		if (newZoom > 10)
			newZoom -= 2;

			diagramView.zoomFactor = newZoom;
	}
}